@@ -173,7 +173,8 @@ typedef enum NMNetworkType
 	NETWORK_TYPE_UNKNOWN = 0,
 	NETWORK_TYPE_ALLOWED,
 	NETWORK_TYPE_INVALID,
-	NETWORK_TYPE_DEVICE
+	NETWORK_TYPE_DEVICE,
+	NETWORK_TYPE_WIRED
 } NMNetworkType;
 
 
