@@ -82,15 +82,18 @@ DBusMessage *nm_dbus_create_error_message (DBusMessage *message, const char *exc
  */
 char * nm_dbus_get_object_path_for_device (NMDevice *dev)
 {
-	char *object_path, *escaped_object_path;
+	char *object_path, *escaped_dev;
 
 	g_return_val_if_fail (dev != NULL, NULL);
 
-	object_path = g_strdup_printf ("%s/%s", NM_DBUS_PATH_DEVICES, nm_device_get_iface (dev));
-	escaped_object_path = nm_dbus_escape_object_path (object_path);
-	g_free (object_path);
+	escaped_dev = nm_dbus_escape_object_path_item (nm_device_get_iface (dev));
+	if (!escaped_dev)
+		return NULL;
 
-	return escaped_object_path;
+	object_path = g_strdup_printf ("%s/%s", NM_DBUS_PATH_DEVICES, escaped_dev);
+	g_free (escaped_dev);
+
+	return object_path;
 }
 
 
@@ -102,7 +105,7 @@ char * nm_dbus_get_object_path_for_device (NMDevice *dev)
  */
 char * nm_dbus_get_object_path_for_network (NMDevice *dev, NMAccessPoint *ap)
 {
-	char *object_path, *escaped_object_path;
+	char *object_path = NULL, *escaped_ssid = NULL, *escaped_dev = NULL;
 
 	g_return_val_if_fail (dev != NULL, NULL);
 	g_return_val_if_fail (ap != NULL, NULL);
@@ -110,11 +113,15 @@ char * nm_dbus_get_object_path_for_network (NMDevice *dev, NMAccessPoint *ap)
 	if (!nm_ap_get_essid (ap))
 		return NULL;
 
-	object_path = g_strdup_printf ("%s/%s/Networks/%s", NM_DBUS_PATH_DEVICES, nm_device_get_iface (dev), nm_ap_get_essid (ap));
-	escaped_object_path = nm_dbus_escape_object_path (object_path);
-	g_free (object_path);
+	escaped_ssid = nm_dbus_escape_object_path_item (nm_ap_get_essid (ap));
+	escaped_dev = nm_dbus_escape_object_path_item (nm_device_get_iface (dev));
+	if (escaped_dev && escaped_ssid)
+		object_path = g_strdup_printf ("%s/%s/Networks/%s", NM_DBUS_PATH_DEVICES, escaped_dev, escaped_ssid);
 
-	return escaped_object_path;
+	g_free (escaped_ssid);
+	g_free (escaped_dev);
+
+	return object_path;
 }
 
 
@@ -139,25 +146,29 @@ NMDevice *nm_dbus_get_device_from_escaped_object_path (NMData *data, const char
 	for (elt = data->dev_list; elt; elt = g_slist_next (elt))
 	{
 		char *compare_path;
-		char *escaped_compare_path;
+		char *escaped_dev;
 		int len;
 
 		if (!(dev = (NMDevice *)(elt->data)))
 			continue;
 
-		compare_path = g_strdup_printf ("%s/%s", NM_DBUS_PATH_DEVICES, nm_device_get_iface (dev));
-		escaped_compare_path = nm_dbus_escape_object_path (compare_path);
-		g_free (compare_path);
-		len = strlen (escaped_compare_path);
+		escaped_dev = nm_dbus_escape_object_path_item (nm_device_get_iface (dev));
+		if (!escaped_dev)
+			continue;
+
+		compare_path = g_strdup_printf ("%s/%s", NM_DBUS_PATH_DEVICES, escaped_dev);
+		g_free (escaped_dev);
+
+		len = strlen (compare_path);
 
 		/* Compare against our constructed path, but ignore any trailing elements */
-		if (    (strncmp (path, escaped_compare_path, len) == 0)
+		if (    (strncmp (path, compare_path, len) == 0)
 			&& ((path[len] == '\0' || path[len] == '/')))
 		{
-			g_free (escaped_compare_path);
+			g_free (compare_path);
 			break;
 		}
-		g_free (escaped_compare_path);
+		g_free (compare_path);
 		dev = NULL;
 	}
 	nm_unlock_mutex (data->dev_list_mutex, __FUNCTION__);
@@ -174,7 +185,7 @@ typedef struct NMStatusChangeData
 {
 	NMData *			data;
 	NMDevice *		dev;
-	NMAccessPoint *	ap;
+	char *essid;
 	DeviceStatus	 	status;
 } NMStatusChangeData;
 
@@ -207,9 +218,9 @@ static DeviceStatusSignals dev_status_signals[] =
 static gboolean nm_dbus_signal_device_status_change (gpointer user_data)
 {
 	NMStatusChangeData *cb_data = (NMStatusChangeData *)user_data;
-	DBusMessage *		message;
-	char *			dev_path;
-	const char *		sig = NULL;
+	DBusMessage *		message = NULL;
+	char *			dev_path = NULL;
+	const char *		sig;
 	int				i = 0;
 
 	g_return_val_if_fail (cb_data->data, FALSE);
@@ -220,36 +231,35 @@ static gboolean nm_dbus_signal_device_status_change (gpointer user_data)
 		i++;
 
 	if (!(sig = dev_status_signals[i].signal))
-		return FALSE;
+		goto out;
 
 	if (!(dev_path = nm_dbus_get_object_path_for_device (cb_data->dev)))
-		return FALSE;
+		goto out;
 
 	if (!(message = dbus_message_new_signal (NM_DBUS_PATH, NM_DBUS_INTERFACE, sig)))
 	{
 		nm_warning ("nm_dbus_signal_device_status_change(): Not enough memory for new dbus message!");
-		g_free (dev_path);
-		return FALSE;
+		goto out;
 	}
 
 	/* If the device was wireless, attach the name of the wireless network that failed to activate */
-	if (cb_data->ap)
+	if (cb_data->essid)
 	{
-		const char *essid = nm_ap_get_essid (cb_data->ap);
-		if (essid)
-			dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_STRING, &essid, DBUS_TYPE_INVALID);
-		nm_ap_unref (cb_data->ap);
+		dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_STRING, &cb_data->essid, DBUS_TYPE_INVALID);
 	}
 	else
 		dbus_message_append_args (message, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_INVALID);
 
-	g_free (dev_path);
-
 	if (!dbus_connection_send (cb_data->data->dbus_connection, message, NULL))
 		nm_warning ("nm_dbus_signal_device_status_change(): Could not raise the signal!");
 
-	dbus_message_unref (message);
+ out:
+	if (message)
+		dbus_message_unref (message);
 
+	g_free (cb_data->essid);
+
+	g_free (dev_path);
 	g_object_unref (G_OBJECT (cb_data->dev));
 	g_free (cb_data);
 
@@ -257,7 +267,7 @@ static gboolean nm_dbus_signal_device_status_change (gpointer user_data)
 }
 
 
-void nm_dbus_schedule_device_status_change_signal (NMData *data, NMDevice *dev, NMAccessPoint *ap, DeviceStatus status)
+void nm_dbus_schedule_device_status_change_signal (NMData *data, NMDevice *dev, const char *essid, DeviceStatus status)
 {
 	NMStatusChangeData	*cb_data = NULL;
 	GSource			*source;
@@ -269,11 +279,9 @@ void nm_dbus_schedule_device_status_change_signal (NMData *data, NMDevice *dev,
 	g_object_ref (G_OBJECT (dev));
 	cb_data->data = data;
 	cb_data->dev = dev;
-	if (ap)
-	{
-		nm_ap_ref (ap);
-		cb_data->ap = ap;
-	}
+	if (essid)
+		cb_data->essid = g_strdup (essid);
+
 	cb_data->status = status;
 
 	source = g_idle_source_new ();
@@ -432,6 +440,29 @@ out:
 }
 
 
+void nm_dbus_signal_wireless_enabled (NMData * data)
+{
+	DBusMessage *	message;
+
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (data->dbus_connection != NULL);
+
+	if (!(message = dbus_message_new_signal (NM_DBUS_PATH, NM_DBUS_INTERFACE, "WirelessEnabled")))
+	{
+		nm_warning ("%s(): Not enough memory for new dbus message!", __func__);
+		return;
+	}
+
+	dbus_message_append_args (message,
+	                          DBUS_TYPE_BOOLEAN, &data->wireless_enabled,
+	                          DBUS_TYPE_BOOLEAN, &data->hw_rf_enabled,
+	                          DBUS_TYPE_INVALID);
+	if (!dbus_connection_send (data->dbus_connection, message, NULL))
+		nm_warning ("%s(): Could not emit the WirelessEnabled signal!", __func__);
+
+	dbus_message_unref (message);
+}
+
 /*
  * nm_dbus_signal_filter
  *
@@ -488,7 +519,7 @@ static DBusHandlerResult nm_dbus_signal_filter (DBusConnection *connection, DBus
 		}
 		else if (dbus_message_is_signal (message, NMI_DBUS_INTERFACE, "UserInterfaceActivated"))
 		{
-			nm_device_802_11_wireless_set_scan_interval (data, NULL, NM_WIRELESS_SCAN_INTERVAL_ACTIVE);
+			nm_device_802_11_wireless_ui_activated (data, NM_WIRELESS_SCAN_INTERVAL_ACTIVE);
 			handled = TRUE;
 		}
 	}
@@ -619,21 +650,24 @@ static DBusHandlerResult nm_dbus_devices_message_handler (DBusConnection *connec
 		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeviceNotFound", "The requested network device does not exist.");
 	else
 	{
-		char			*object_path, *escaped_object_path;
+		char			*object_path, *escaped_dev;
 		NMDbusCBData	 cb_data;
 
 		cb_data.data = data;
 		cb_data.dev = dev;
 
 		/* Test whether or not the _networks_ of a device were queried instead of the device itself */
-		object_path = g_strdup_printf ("%s/%s/Networks/", NM_DBUS_PATH_DEVICES, nm_device_get_iface (dev));
-		escaped_object_path = nm_dbus_escape_object_path (object_path);
-		g_free (object_path);
-		if (strncmp (path, escaped_object_path, strlen (escaped_object_path)) == 0)
-			handled = nm_dbus_method_dispatch (data->net_methods, connection, message, &cb_data, &reply);
-		else
-			handled = nm_dbus_method_dispatch (data->device_methods, connection, message, &cb_data, &reply);
-		g_free (escaped_object_path);
+		escaped_dev = nm_dbus_escape_object_path_item (nm_device_get_iface (dev));
+		if (escaped_dev) {
+			object_path = g_strdup_printf ("%s/%s/Networks/", NM_DBUS_PATH_DEVICES, escaped_dev);
+			g_free (escaped_dev);
+
+			if (strncmp (path, object_path, strlen (object_path)) == 0)
+				handled = nm_dbus_method_dispatch (data->net_methods, connection, message, &cb_data, &reply);
+			else
+				handled = nm_dbus_method_dispatch (data->device_methods, connection, message, &cb_data, &reply);
+			g_free (object_path);
+		}
 	}
 
 	if (reply)
@@ -831,10 +865,10 @@ DBusConnection *nm_dbus_init (NMData *data)
 	}
 
 	dbus_error_init (&error);
-#if (DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR >= 60)
-	flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;	/* Prohibit replacement is now the default */
-#else
+#if (DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 60)
 	flags = DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT;
+#else
+	flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
 #endif
 	ret = dbus_bus_request_name (connection, NM_DBUS_SERVICE, flags, &error);
 	if (dbus_error_is_set (&error))
