@@ -51,7 +51,8 @@ static gboolean nm_policy_activation_finish (NMActRequest *req)
 {
 	NMDevice			*dev = NULL;
 	NMData			*data = NULL;
-	NMAccessPoint *	ap = NULL;
+	NMActRequest * dev_req;
+	const char *network_id = NULL;
 
 	g_return_val_if_fail (req != NULL, FALSE);
 
@@ -61,13 +62,26 @@ static gboolean nm_policy_activation_finish (NMActRequest *req)
 	dev = nm_act_request_get_dev (req);
 	g_assert (dev);
 
-    if (nm_device_is_802_11_wireless (dev))
-        ap = nm_act_request_get_ap (req);
+	/* Ensure that inactive devices don't get the activated signal
+	 * sent due to race conditions.
+	 */
+	dev_req = nm_device_get_act_request (dev);
+	if (!dev_req || (dev_req != req))
+		return FALSE;
+
+    if (NM_IS_DEVICE_802_11_WIRELESS (dev))
+		network_id = nm_ap_get_essid (nm_act_request_get_ap (req));
+	else if (NM_IS_DEVICE_802_3_ETHERNET (dev)) {
+		NMWiredNetwork *wired_net = nm_act_request_get_wired_network (req);
+		if (wired_net)
+			network_id = nm_wired_network_get_network_id (wired_net);
+	}
 
 	nm_device_activation_success_handler (dev, req);
 
+	nm_act_request_unref (req);
 	nm_info ("Activation (%s) successful, device activated.", nm_device_get_iface (dev));
-	nm_dbus_schedule_device_status_change_signal (data, dev, ap, DEVICE_NOW_ACTIVE);
+	nm_dbus_schedule_device_status_change_signal (data, dev, network_id, DEVICE_NOW_ACTIVE);
 	nm_schedule_state_change_signal_broadcast (data);
 
 	return FALSE;
@@ -93,6 +107,7 @@ void nm_policy_schedule_activation_finish (NMActRequest *req)
 	g_assert (dev);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_ACTIVATED);
+	nm_act_request_ref (req);
 
 	source = g_idle_source_new ();
 	g_source_set_priority (source, G_PRIORITY_HIGH_IDLE);
@@ -113,7 +128,7 @@ static gboolean nm_policy_activation_failed (NMActRequest *req)
 {
 	NMDevice *	dev = NULL;
 	NMData *		data = NULL;
-	NMAccessPoint *ap = NULL;
+	const char *network_id = NULL;
 
 	g_return_val_if_fail (req != NULL, FALSE);
 
@@ -125,16 +140,23 @@ static gboolean nm_policy_activation_failed (NMActRequest *req)
 
 	nm_device_activation_failure_handler (dev, req);
 
-    if (nm_device_is_802_11_wireless (dev))
-        ap = nm_act_request_get_ap (req);
+	if (NM_IS_DEVICE_802_11_WIRELESS (dev))
+		network_id = nm_ap_get_essid (nm_act_request_get_ap (req));
+	else if (NM_IS_DEVICE_802_3_ETHERNET (dev)) {
+		NMWiredNetwork *wired_net = nm_act_request_get_wired_network (req);
+		if (wired_net)
+			network_id = nm_wired_network_get_network_id (wired_net);
+	}
 
 	nm_info ("Activation (%s) failed.", nm_device_get_iface (dev));
-	nm_dbus_schedule_device_status_change_signal	(data, dev, ap, DEVICE_ACTIVATION_FAILED);
+	nm_dbus_schedule_device_status_change_signal (data, dev, network_id, DEVICE_ACTIVATION_FAILED);
 
 	nm_device_deactivate (dev);
 	nm_schedule_state_change_signal_broadcast (data);
 	nm_policy_schedule_device_change_check (data);
 
+	nm_act_request_unref (req);
+
 	return FALSE;
 }
 
@@ -158,6 +180,7 @@ void nm_policy_schedule_activation_failed (NMActRequest *req)
 	g_assert (dev);
 
 	nm_act_request_set_stage (req, NM_ACT_STAGE_FAILED);
+	nm_act_request_ref (req);
 
 	source = g_idle_source_new ();
 	g_source_set_priority (source, G_PRIORITY_HIGH_IDLE);
@@ -260,14 +283,16 @@ static NMDevice * nm_policy_auto_get_best_device (NMData *data, NMAccessPoint **
 	}
 
 #if 0
-	nm_info ("AUTO: Best wired device = %s, best wireless device = %s (%s)", best_wired_dev ? nm_device_get_iface (best_wired_dev) : "(null)",
-			best_wireless_dev ? nm_device_get_iface (best_wireless_dev) : "(null)", (best_wireless_dev && *ap) ? nm_ap_get_essid (*ap) : "null" );
+	nm_info ("AUTO: Best wired device = %s, best wireless device = %s (%s)", best_wired_dev ? nm_device_get_iface (NM_DEVICE (best_wired_dev)) : "(null)",
+			best_wireless_dev ? nm_device_get_iface (NM_DEVICE (best_wireless_dev)) : "(null)", (best_wireless_dev && *ap) ? nm_ap_get_essid (*ap) : "null" );
 #endif
 
 	return highest_priority_dev;
 }
 
 
+static GStaticMutex dcc_mutex = G_STATIC_MUTEX_INIT;
+
 /*
  * nm_policy_device_change_check
  *
@@ -289,7 +314,9 @@ nm_policy_device_change_check (NMData *data)
 
 	g_return_val_if_fail (data != NULL, FALSE);
 
+	g_static_mutex_lock (&dcc_mutex);
 	data->dev_change_check_idle_id = 0;
+	g_static_mutex_unlock (&dcc_mutex);
 
 	old_dev = nm_get_active_device (data);
 
@@ -298,11 +325,19 @@ nm_policy_device_change_check (NMData *data)
 
 	if (old_dev)
 	{
+		gboolean has_link = TRUE;
 		guint32 caps = nm_device_get_capabilities (old_dev);
 
+		/* Ensure ethernet devices have a link before starting activation,
+		 * partially works around Fedora #194124.
+		 */
+		if (nm_device_is_802_3_ethernet (old_dev))
+			has_link = nm_device_has_active_link (old_dev);
+
 		/* Don't interrupt a currently activating device. */
 		if (   nm_device_is_activating (old_dev)
-		    && !nm_device_can_interrupt_activation (old_dev))
+		    && !nm_device_can_interrupt_activation (old_dev)
+		    && has_link)
 		{
 			nm_info ("Old device '%s' activating, won't change.", nm_device_get_iface (old_dev));
 			goto out;
@@ -311,13 +346,10 @@ nm_policy_device_change_check (NMData *data)
 		/* Don't interrupt semi-supported devices either.  If the user chose one, they must
 		 * explicitly choose to move to another device, we're not going to move for them.
 		 */
-		if ((nm_device_is_802_3_ethernet (old_dev) && !(caps & NM_DEVICE_CAP_CARRIER_DETECT))
-			|| (nm_device_is_802_11_wireless (old_dev) && !(caps & NM_DEVICE_CAP_WIRELESS_SCAN)))
-		{
-			nm_info ("Old device '%s' was semi-supported and user chosen, won't change unless told to.",
-				nm_device_get_iface (old_dev));
+		if ((nm_device_is_802_3_ethernet (old_dev) && !(caps & NM_DEVICE_CAP_CARRIER_DETECT)))
+			goto out;
+		if (nm_device_is_802_11_wireless (old_dev) && !(caps & NM_DEVICE_CAP_WIRELESS_SCAN))
 			goto out;
-		}
 	}
 
 	new_dev = nm_policy_auto_get_best_device (data, &ap);
@@ -423,11 +455,39 @@ nm_policy_device_change_check (NMData *data)
 	if (do_switch && (nm_device_is_802_3_ethernet (new_dev) || (nm_device_is_802_11_wireless (new_dev) && ap)))
 	{
 		NMActRequest *	act_req = NULL;
+		gboolean has_link = TRUE;
 
-		if ((act_req = nm_act_request_new (data, new_dev, ap, FALSE)))
+		/* Ensure ethernet devices have a link before starting activation,
+		 * partially works around Fedora #194124.
+		 */
+		if (nm_device_is_802_3_ethernet (new_dev))
+			has_link = nm_device_has_active_link (new_dev);
+
+		if (has_link)
 		{
-			nm_info ("Will activate connection '%s%s%s'.", nm_device_get_iface (new_dev), ap ? "/" : "", ap ? nm_ap_get_essid (ap) : "");
-			nm_policy_schedule_device_activation (act_req);
+			if ((act_req = nm_act_request_new (data, new_dev, FALSE)))
+			{
+				nm_info ("Will activate connection '%s%s%s'.",
+				         nm_device_get_iface (new_dev),
+				         ap ? "/" : "",
+				         ap ? nm_ap_get_essid (ap) : "");
+
+				if (ap)
+					nm_act_request_set_ap (act_req, ap);
+
+				nm_policy_schedule_device_activation (act_req);
+				nm_act_request_unref (act_req);
+			}
+			else
+			{
+				nm_info ("Error creating activation request for %s",
+				         nm_device_get_iface (new_dev));
+			}
+		}
+		else
+		{
+			nm_info ("Won't activate %s because it no longer has a link.",
+			         nm_device_get_iface (new_dev));
 		}
 	}
 
@@ -449,11 +509,9 @@ out:
  */
 void nm_policy_schedule_device_change_check (NMData *data)
 {
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-
 	g_return_if_fail (data != NULL);
 
-	g_static_mutex_lock (&mutex);
+	g_static_mutex_lock (&dcc_mutex);
 
 	if (data->dev_change_check_idle_id == 0)
 	{
@@ -463,8 +521,7 @@ void nm_policy_schedule_device_change_check (NMData *data)
 		data->dev_change_check_idle_id = g_source_attach (source, data->main_context);
 		g_source_unref (source);
 	}
-
-	g_static_mutex_unlock (&mutex);
+	g_static_mutex_unlock (&dcc_mutex);
 }
 
 
@@ -490,10 +547,11 @@ static gboolean nm_policy_device_activation (NMActRequest *req)
 		nm_device_deactivate (old_dev);
 
 	new_dev = nm_act_request_get_dev (req);
-	if (nm_device_is_activating (new_dev))
-		return FALSE;
 
-	nm_device_activation_start (req);
+	if (!nm_device_is_activating (new_dev))
+		nm_device_activation_start (req);
+
+	nm_act_request_unref (req);
 
 	return FALSE;
 }
@@ -519,6 +577,8 @@ void nm_policy_schedule_device_activation (NMActRequest *req)
 	dev = nm_act_request_get_dev (req);
 	g_assert (dev);
 
+	nm_act_request_ref (req);
+
 	source = g_idle_source_new ();
 	g_source_set_priority (source, G_PRIORITY_HIGH_IDLE);
 	g_source_set_callback (source, (GSourceFunc) nm_policy_device_activation, req, NULL);
