@@ -23,11 +23,18 @@
  * (C) Copyright 2005-2006 SuSE GmbH
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <signal.h>
 #include <sys/stat.h>
 #include <arpa/inet.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <string.h>
 
 #include "NetworkManagerSystem.h"
 #include "NetworkManagerUtils.h"
@@ -94,7 +101,7 @@ void nm_system_device_flush_routes_with_iface (const char *iface)
 	g_return_if_fail (iface != NULL);
 
 	/* Remove routing table entries */
-	buf = g_strdup_printf ("/sbin/ip route flush dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route flush dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -131,7 +138,7 @@ void nm_system_device_add_default_route_via_device_with_iface (const char *iface
 	g_return_if_fail (iface != NULL);
 
 	/* Add default gateway */
-	buf = g_strdup_printf ("/sbin/ip route add default dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route add default dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -150,7 +157,7 @@ void nm_system_device_add_route_via_device_with_iface (const char *iface, const
 	g_return_if_fail (iface != NULL);
 
 	/* Add default gateway */
-	buf = g_strdup_printf ("/sbin/ip route add %s dev %s", route, iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " route add %s dev %s", route, iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -200,7 +207,7 @@ void nm_system_device_flush_addresses_with_iface (const char *iface)
 	g_return_if_fail (iface != NULL);
 
 	/* Remove all IP addresses for a device */
-	buf = g_strdup_printf ("/sbin/ip addr flush dev %s", iface);
+	buf = g_strdup_printf (IP_BINARY_PATH " addr flush dev %s", iface);
 	nm_spawn_process (buf);
 	g_free (buf);
 }
@@ -215,7 +222,7 @@ void nm_system_device_flush_addresses_with_iface (const char *iface)
 void nm_system_enable_loopback (void)
 {
 	nm_system_device_set_up_down_with_iface ("lo", TRUE);
-	nm_spawn_process ("/sbin/ip addr add 127.0.0.1/8 brd 127.255.255.255 dev lo scope host label loopback");
+	nm_spawn_process (IP_BINARY_PATH " addr add 127.0.0.1/8 brd 127.255.255.255 dev lo scope host label loopback");
 }
 
 
@@ -240,7 +247,7 @@ void nm_system_flush_loopback_routes (void)
  */
 void nm_system_delete_default_route (void)
 {
-	nm_spawn_process ("/sbin/ip route del default");
+	nm_spawn_process (IP_BINARY_PATH " route del default");
 }
 
 
@@ -252,7 +259,7 @@ void nm_system_delete_default_route (void)
  */
 void nm_system_flush_arp_cache (void)
 {
-	nm_spawn_process ("/sbin/ip neigh flush all");
+	nm_spawn_process (IP_BINARY_PATH " neigh flush all");
 }
 
 
@@ -276,15 +283,8 @@ void nm_system_kill_all_dhcp_daemons (void)
  */
 void nm_system_update_dns (void)
 {
-#ifdef NM_NO_NAMED
-	if (nm_spawn_process (SYSCONFDIR"/init.d/nscd status") != 0)
-		nm_spawn_process (SYSCONFDIR"/init.d/nscd restart");
-
 	nm_info ("Clearing nscd hosts cache.");
 	nm_spawn_process ("/usr/sbin/nscd -i hosts");
-#else
-	nm_spawn_process ("/usr/bin/killall -q nscd");
-#endif
 }
 
 
@@ -328,11 +328,7 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	struct ether_addr hw_addr;
 	unsigned char eui[8];
 
-	if (nm_device_is_802_3_ethernet (dev))
-		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
-	else if (nm_device_is_802_11_wireless (dev))
-		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
-
+	nm_device_get_hw_address (dev, &hw_addr);
 	memcpy (eui, &(hw_addr.ether_addr_octet), sizeof (hw_addr.ether_addr_octet));
 	memmove (eui+5, eui+3, 3);
 	eui[3] = 0xff;
@@ -340,7 +336,7 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	eui[0] ^= 2;
 
 	/* Add the default link-local IPv6 address to a device */
-	buf = g_strdup_printf ("/sbin/ip -6 addr add fe80::%x%02x:%x%02x:%x%02x:%x%02x/64 dev %s",
+	buf = g_strdup_printf (IP_BINARY_PATH " -6 addr add fe80::%x%02x:%x%02x:%x%02x:%x%02x/64 dev %s",
 						eui[0], eui[1], eui[2], eui[3], eui[4], eui[5],
 						eui[6], eui[7], nm_device_get_iface (dev));
 	nm_spawn_process (buf);
@@ -432,6 +428,38 @@ out:
 }
 
 
+static gboolean
+parse_netmask (const char *buf, guint32 *netmask)
+{
+	gboolean success = FALSE;
+
+	if (strstr (buf, ".")) {
+		struct in_addr addr;
+
+		if (inet_aton (buf, &addr)) {
+			*netmask = addr.s_addr;
+			success = TRUE;
+		}
+	} else {
+		int maskval;
+
+		maskval = atoi (buf);
+		if (maskval >= 0 && maskval <= 32) {
+			guint32 mask = 0;
+
+			while (maskval > 0) {
+				mask = (mask >> 1) + 0x80000000;
+				maskval--;
+			}
+
+			*netmask = htonl (mask);
+			success = TRUE;
+		}
+	}
+
+	return success;
+}
+
 /*
  * nm_system_device_get_system_config
  *
@@ -461,11 +489,7 @@ void *nm_system_device_get_system_config (NMDevice *dev, NMData *app_data)
 	sys_data = g_malloc0 (sizeof (SuSEDeviceConfigData));
 	sys_data->use_dhcp = TRUE;
 
-	if (nm_device_is_802_3_ethernet (dev))
-		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
-	else if (nm_device_is_802_11_wireless (dev))
-		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
-
+	nm_device_get_hw_address (dev, &hw_addr);
 	sprintf (mac, "%02x:%02x:%02x:%02x:%02x:%02x",
 			hw_addr.ether_addr_octet[0], hw_addr.ether_addr_octet[1],
 			hw_addr.ether_addr_octet[2], hw_addr.ether_addr_octet[3],
@@ -662,6 +686,8 @@ found:
 		free (mode);
 		free (buf);
 	}
+	else if (buf)
+		g_free (buf);
 
 	sys_data->config = nm_ip4_config_new ();
 
@@ -670,14 +696,38 @@ found:
 		buf = svGetValue (file, "IPADDR");
 		if (buf)
 		{
-			struct in_addr ip;
-			int ret;
+			char **pieces;
+			char *ip_str = NULL;
+			char *netmask_str = NULL;
 
-			ret = inet_aton (buf, &ip);
-			if (ret)
-				nm_ip4_config_set_address (sys_data->config, ip.s_addr);
-			else
+			pieces = g_strsplit (buf, "/", 0);
+			if (g_strv_length (pieces) == 1) {
+				ip_str = buf;
+			} else if (g_strv_length (pieces) == 2) {
+				ip_str = pieces[0];
+				netmask_str = pieces[1];
+			} else
 				error = TRUE;
+
+			if (!error && ip_str) {
+				struct in_addr ip;
+
+				if (inet_aton (ip_str, &ip))
+					nm_ip4_config_set_address (sys_data->config, ip.s_addr);
+				else
+					error = TRUE;
+			}
+
+			if (!error && netmask_str) {
+				guint32 netmask;
+
+				if (parse_netmask (netmask_str, &netmask))
+					nm_ip4_config_set_netmask (sys_data->config, netmask);
+				else
+					error = TRUE;
+			}
+
+			g_strfreev (pieces);
 			free (buf);
 		}
 		else
@@ -692,7 +742,8 @@ found:
 
 		if ((buf = svGetValue (file, "NETMASK")))
 		{
-			nm_ip4_config_set_netmask (sys_data->config, inet_addr (buf));
+			if (nm_ip4_config_get_netmask (sys_data->config) == 0)
+				nm_ip4_config_set_netmask (sys_data->config, inet_addr (buf));
 			free (buf);
 		}
 		else
@@ -811,6 +862,8 @@ void nm_system_device_free_system_config (NMDevice *dev, void *system_config_dat
 
 	if (sys_data->config)
 		nm_ip4_config_unref (sys_data->config);
+
+	g_free (sys_data);
 }
 
 
@@ -1023,13 +1076,15 @@ GSList * nm_system_get_dialup_config (void)
 		NMDialUpConfig *config;
 		shvarFile *modem_file;
 		char *name, *buf, *provider_name;
-		int modem;
+		int type;
 
 		/* we only want modems and isdn */
 		if (g_str_has_prefix (dentry, "ifcfg-modem"))
-			modem = 1;
+			type = NM_DIALUP_TYPE_MODEM;
 		else if (g_str_has_prefix (dentry, "ifcfg-ippp"))
-			modem = 0;
+			type = NM_DIALUP_TYPE_ISDN;
+		else if (g_str_has_prefix (dentry, "ifcfg-dsl"))
+			type = NM_DIALUP_TYPE_DSL;
 		else
 			continue;
 
@@ -1049,15 +1104,21 @@ GSList * nm_system_get_dialup_config (void)
 
 		config = g_malloc (sizeof (NMDialUpConfig));
 		config->data = g_strdup (dentry + 6); /* skip the "ifcfg-" prefix */
-		if (modem)
+		if (type == NM_DIALUP_TYPE_MODEM)
 		{
 			config->name = g_strdup_printf ("%s via modem (%s)", provider_name, (char *) config->data);
 			config->type = NM_DIALUP_TYPE_MODEM;
-		} else
+		}
+		else if (type == NM_DIALUP_TYPE_ISDN)
 		{
 			config->name = g_strdup_printf ("%s via ISDN (%s)", provider_name, (char *) config->data);
 			config->type = NM_DIALUP_TYPE_ISDN;
 		}
+		else if (type == NM_DIALUP_TYPE_DSL)
+		{
+			config->name = g_strdup_printf ("%s via DSL (%s)", provider_name, (char *) config->data);
+			config->type = NM_DIALUP_TYPE_DSL;
+		}
 
 		list = g_slist_append (list, config);
 
@@ -1138,11 +1199,6 @@ void nm_system_activate_nis (NMIP4Config *config)
 			} else
 				nm_warning ("Could not commit NIS changes to /etc/yp.conf.");
 
-			if (stat ("/usr/sbin/rcypbind", &sb) != -1)
-			{
-				nm_info ("Restarting ypbind.");
-				nm_spawn_process ("/usr/sbin/rcypbind reload");
-			}
 			if (stat ("/usr/sbin/rcautofs", &sb) != -1)
 			{
 				nm_info ("Restarting autofs.");
@@ -1167,41 +1223,6 @@ out_gfree:
  */
 void nm_system_shutdown_nis (void)
 {
-#if 0	/* XXX: let's not touch NIS, for now; probably need to make this a configurable option */
-	char *name, *buf = NULL;
-	shvarFile *file;
-
-	name = g_strdup_printf (SYSCONFDIR"/sysconfig/network/dhcp");
-	file = svNewFile (name);
-	if (!file)
-		goto out_gfree;
-
-	buf = svGetValue (file, "DHCLIENT_MODIFY_NIS_CONF");
-	if (!buf)
-		goto out_close;
-
-	if (!strcmp (buf, "yes")) {
-		struct stat sb;
-
-		if (stat ("/usr/sbin/rcypbind", &sb) != -1)
-		{
-			nm_info ("Stopping ypbind.");
-			nm_spawn_process ("/usr/sbin/rcypbind stop");
-		}
-
-		if (stat ("/usr/sbin/rcautofs", &sb) != -1)
-		{
-			nm_info ("Restarting autofs.");
-			nm_spawn_process ("/usr/sbin/rcautofs restart");
-		}
-	}
-
-	free (buf);
-out_close:
-	svCloseFile (file);
-out_gfree:
-	g_free (name);
-#endif
 }
 
 
