@@ -620,7 +620,7 @@ nm_device_set_active_link (NMDevice *self,
 				do_switch = TRUE;
 
 			/* FIXME: Why is this activation request created here and never used? */
-			/* if (do_switch && (act_req = nm_act_request_new (app_data, self, NULL, TRUE))) */
+			/* if (do_switch && (act_req = nm_act_request_new (app_data, self, TRUE))) */
 			if (do_switch)
 			{
 				nm_info ("Will activate wired connection '%s' because it now has a link.", nm_device_get_iface (self));
@@ -856,6 +856,12 @@ real_act_stage3_ip_config_start (NMDevice *self,
 	data = nm_act_request_get_data (req);
 	g_assert (data);
 
+	/* Sometimes the device gets downed by wpa_supplicant; in any case, make
+	 * sure it's up before anything tries to use it.
+	 */
+	if (!nm_device_is_up (self))
+		nm_device_bring_up (self);
+
 	/* DHCP devices try DHCP, non-DHCP default to SUCCESS */
 	if (nm_device_get_use_dhcp (self))
 	{
