@@ -25,6 +25,7 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <dbus/dbus.h>
+#include <unistd.h>
 #include "NetworkManagerDbus.h"
 #include "nm-dbus-vpn.h"
 #include "nm-vpn-service.h"
@@ -297,6 +298,21 @@ void nm_vpn_service_start_connection (NMVPNService *service, NMVPNActRequest *re
 }
 
 
+/*
+ * nm_vpn_service_child_setup
+ *
+ * Set the process group ID of the newly forked process
+ *
+ */
+static void
+nm_vpn_service_child_setup (gpointer user_data G_GNUC_UNUSED)
+{
+	/* We are in the child process at this point */
+	pid_t pid = getpid ();
+	setpgid (pid, pid);
+}
+
+
 /*
  * nm_vpn_service_stage_1_daemon_exec
  *
@@ -327,7 +343,7 @@ static gboolean nm_vpn_service_stage1_daemon_exec (gpointer user_data)
 	g_ptr_array_add (vpn_argv, service->program);
 	g_ptr_array_add (vpn_argv, NULL);
 
-	if (!g_spawn_async (NULL, (char **) vpn_argv->pdata, NULL, 0, NULL, NULL, &pid, &error))
+	if (!g_spawn_async (NULL, (char **) vpn_argv->pdata, NULL, 0, &nm_vpn_service_child_setup, NULL, &pid, &error))
 	{
 		g_ptr_array_free (vpn_argv, TRUE);
 		nm_warning ("(VPN Service %s): could not launch the VPN service.  error: '%s'.", service->service, error->message);
@@ -654,8 +670,8 @@ static void nm_vpn_service_schedule_stage4_ip_config_get_timeout (NMVPNService *
 
 	nm_vpn_act_request_set_stage (req, NM_VPN_ACT_STAGE_IP_CONFIG_GET);
 
-	/* 20 second timeout waiting for IP config signal from VPN service */
-	source = g_timeout_source_new (20000);
+	/* 45 second timeout waiting for IP config signal from VPN service */
+	source = g_timeout_source_new (45000);
 	g_source_set_callback (source, (GSourceFunc) nm_vpn_service_stage4_ip_config_get_timeout, req, NULL);
 	id = g_source_attach (source, service->app_data->main_context);
 	nm_vpn_act_request_set_callback_id (req, id);
